struct SSets
{
  string name;
  MqlTick ticks[];
  MqlRates rates[];
};

struct SLevel 
{
  int index;
  double up;
  double down;
};

class CHelpers
{
  public : 

    int pointPrecision;
    int pricePrecision;
    int averageCount[];

    double averageSummary[];

    CHelpers() 
    {
      pointPrecision = 5;
      pricePrecision = 2;
    }

    double pipsToMoney(
      const string symbol, 
      const double pips)
    {
      double step = SymbolInfoDouble(symbol, SYMBOL_TRADE_TICK_SIZE);
      double value = SymbolInfoDouble(symbol, SYMBOL_TRADE_TICK_VALUE);
      return NormalizeDouble(pips * value / step, pricePrecision);
    }

    double moneyToPips(
      const string symbol, 
      const double amount)
    {
      double step = SymbolInfoDouble(symbol, SYMBOL_TRADE_TICK_SIZE);
      double value = SymbolInfoDouble(symbol, SYMBOL_TRADE_TICK_VALUE);

      if (value == 0)
      {
        return 0;
      }

      return NormalizeDouble(amount * step / value, pointPrecision);
    }

    double getPrice(
      const string symbol, 
      const ENUM_TIMEFRAMES period = PERIOD_CURRENT, 
      const int position = 0)
    {
      MqlRates bars[];

      if (CopyRates(symbol, period, position, 1, bars))
      {
        return NormalizeDouble(bars[0].close, pointPrecision);
      }

      return 0;
    }

    double getCurrentPrice(const string symbol, const int direction = 0)
    {
      switch (direction)
      {
        case 1: return SymbolInfoDouble(symbol, SYMBOL_ASK); break;
        case -1: return SymbolInfoDouble(symbol, SYMBOL_BID); break;
      }
      
      return (SymbolInfoDouble(symbol, SYMBOL_ASK) + SymbolInfoDouble(symbol, SYMBOL_BID)) / 2;
    }
    
    double getValue(
      const string symbol,
      const double currentPrice = 0)
    {
      double symbolPrice = currentPrice;
      double symbolTick = SymbolInfoDouble(Symbol(), SYMBOL_TRADE_TICK_SIZE);
      double symbolValue = SymbolInfoDouble(Symbol(), SYMBOL_TRADE_TICK_VALUE);
      
      if (currentPrice == 0)
      {
        symbolPrice = SymbolInfoDouble(symbol, SYMBOL_ASK);
      }
      
      return symbolPrice / symbolTick * symbolValue;
    }
    
    double getMargin(
      const string symbol, 
      const double volume,
      const double currentPrice = 0)
    {
      double symbolPrice = currentPrice;
      double symbolMarginInitialRate = 1;
      double symbolMarginMaintenanceRate = 1;
      double symbolTick = SymbolInfoDouble(symbol, SYMBOL_TRADE_TICK_SIZE);
      double symbolValue = SymbolInfoDouble(symbol, SYMBOL_TRADE_TICK_VALUE);
      double symbolContract = SymbolInfoDouble(symbol, SYMBOL_TRADE_CONTRACT_SIZE);
      double accountLeverage = (double) AccountInfoInteger(ACCOUNT_LEVERAGE);
    
      SymbolInfoMarginRate(symbol, ORDER_TYPE_BUY, symbolMarginInitialRate, symbolMarginMaintenanceRate);
      
      if (currentPrice == 0)
      {
        symbolPrice = SymbolInfoDouble(symbol, SYMBOL_ASK);
      }
    
      double margin = 0;
      double symbolMarginRate = symbolMarginInitialRate;
      double symbolMarginInitial = SymbolInfoDouble(symbol, SYMBOL_MARGIN_INITIAL);
      double symbolMarginMaintenance = SymbolInfoDouble(symbol, SYMBOL_MARGIN_MAINTENANCE);
    
      if (symbolMarginInitial == 0) 
      {
        symbolMarginInitial = symbolPrice * symbolMarginInitialRate;
      }
    
      if (symbolMarginMaintenance == 0) 
      {
        symbolMarginMaintenance = symbolPrice * symbolMarginMaintenanceRate;
      }
    
      switch (int(SymbolInfoInteger(symbol, SYMBOL_TRADE_CALC_MODE)))
      {
        case SYMBOL_CALC_MODE_FOREX: 
          margin = volume * symbolPrice * symbolMarginRate * symbolValue / symbolTick / accountLeverage; 
          break;
        
        case SYMBOL_CALC_MODE_EXCH_STOCKS: 
          margin = volume * symbolPrice * symbolMarginRate * symbolValue / symbolTick; 
          break;
          
        case SYMBOL_CALC_MODE_FUTURES:
        case SYMBOL_CALC_MODE_EXCH_FUTURES: 
          margin = volume * symbolMarginInitial * symbolMarginRate; 
          break;
        
        case SYMBOL_CALC_MODE_CFD: 
          margin = volume * symbolPrice * symbolMarginRate * symbolValue / symbolTick / accountLeverage; 
          break;
      }  
      
      return margin;
    }

    bool isTradingTime(
      const int startHour,
      const int startMinute,
      const int endHour,
      const int endMinute)
    {
      MqlDateTime date = {0};
      TimeCurrent(date);

      return 
        date.hour >= startHour && 
        date.min >= startMinute && 
        date.hour <= endHour && 
        date.min <= endMinute;
    }

    double EMA(
      const double current, 
      const double previous, 
      const int period)
    {
      double result = 0.0;
  
      if (period > 0)
      {
        double pr = 2.0 / (period + 1.0);
        return current * pr + previous * (1 - pr);
      }
  
      return current;
    }
    
    void debug(
      const string data, 
      const string name = NULL)
    {
      int descriptor = FileOpen(name == NULL ? IntegerToString(TimeLocal()) : name, 
        FILE_SHARE_READ | 
        FILE_WRITE | 
        FILE_READ | 
        FILE_ANSI);

      FileSeek(descriptor, 0, SEEK_END);      
      FileWrite(descriptor, data);
      FileFlush(descriptor);
      FileClose(descriptor);
    }

    bool isNextBar()
    {
      static datetime previous = 0;
      datetime current = (datetime) SeriesInfoInteger(Symbol(), PERIOD_CURRENT, SERIES_LASTBAR_DATE);

      if (current > previous)
      {
        previous = current;
        return true;
      }
      
      return false;
    }

    int getRandom(const bool binary)
    {
      int number = 1 + INT_MAX * MathRand() / INT_MAX;
  
      if (binary)
      {
        return number % 2 == 0 ? 1 : -1;
      }
  
      return number;
    }

    int getPrecision(const string symbol)
    {
      return (int) SymbolInfoInteger(symbol, SYMBOL_DIGITS);
    }

    MqlRates getBarData(
      const string symbol, 
      const ENUM_TIMEFRAMES period = NULL, 
      const int index = 0)
    {
      MqlRates rate = {0};
      MqlRates rates[];

      if (CopyRates(symbol, period, index, 1, rates))
      {
        return rates[0];
      }

      return rate;
    }

    double getAverage(
      const double average, 
      const double value, 
      const int size)
    {
      return (size * average + value) / (size + 1);
    }

    double getAverage(
      const double value, 
      const int pointer = 0,
      const int calculator = 0)
    {
      int index = ArraySize(averageCount);

      if (index <= pointer)
      {
        ArrayResize(averageCount, pointer + 1);
        ArrayResize(averageSummary, pointer + 1);
      }

      if (calculator > 0)
      {
        averageCount[pointer]++;
        averageSummary[pointer] += value;
      }

      return averageSummary[pointer] / MathMax(averageCount[pointer], 1);
    }

    int getQuotes(
      SSets& series, 
      const int position, 
      const int count,
      const ENUM_TIMEFRAMES period = PERIOD_CURRENT)
    {
      int items = CopyRates(series.name, period, position, count, series.rates);

      ArraySetAsSeries(series.rates, true);

      if (items < 1)
      {
        Print(
          "Synchronization : " + series.name + ", " + 
          "Position : " + IntegerToString(position) + ", " + 
          "Depth : " + IntegerToString(count));

        return 0;
      }

      return items;
    }

    int getTicks(
      SSets& series, 
      const ulong start = 0, 
      const ulong end = 0,
      const bool any = false)
    {
      int items = CopyTicksRange(series.name, series.ticks, COPY_TICKS_ALL, start, end);

      if (any && items < 1)
      {
        items = CopyTicks(series.name, series.ticks, COPY_TICKS_ALL, 0, 100000);
      }

      ArraySetAsSeries(series.ticks, true);

      return items;
    }

    int getFractal(
      const SSets &s, 
      const int index, 
      const int count, 
      const int countL, 
      const int countR)
    {
      int max = MathMax(countL, countR);
      double curDn = s.rates[index].low;
      double curUp = s.rates[index].high;
    
      bool isUpL = true;
      bool isUpR = true;
      bool isDnL = true;
      bool isDnR = true;
    
      for (int i = 1; i <= max; i++) 
      {
        if (i <= countL)
        {
          int nxt = index + i;

          if (nxt >= count - 1)
          {
            return 0;
          }

          double nxtDn = s.rates[nxt].low;
          double nxtUp = s.rates[nxt].high;
    
          isUpL = isUpL && curUp >= nxtUp; 
          isDnL = isDnL && curDn <= nxtDn; 
        }
    
        if (i <= countR)
        {
          int prv = index - i;

          if (prv <= 0)
          {
            return 0;
          }

          double prvDn = s.rates[prv].low;
          double prvUp = s.rates[prv].high;
    
          isUpR = isUpR && curUp >= prvUp; 
          isDnR = isDnR && curDn <= prvDn; 
        }
      }
    
      if (isUpL && isUpR)
      {
        return 1;
      }
    
      if (isDnL && isDnR)
      {
        return -1;
      }  
    
      return 0;
    }
    
    int getLevels(
      SLevel &levels[], 
      const string symbol, 
      const int count, 
      const int countL, 
      const int countR)
    {
      SSets s;

      ZeroMemory(s);

      s.name = symbol;
    
      int bars = getQuotes(s, 0, count);

      ArraySetAsSeries(s.rates, true);

      int levelsIndex = -1;
      int previousDirection = 0;
      int prices = ArraySize(s.rates);

      ArrayResize(levels, prices);
      ZeroMemory(levels);
      
      double prvUp = -INT_MAX;
      double prvDn = INT_MAX;

      for (int i = 1; i < prices; i++) 
      {
        int direction = getFractal(s, i, prices, countL, countR);

        if (direction != 0) 
        {
          double curUp = s.rates[i].high;
          double curDn = s.rates[i].low;

          if (direction > 0) 
          {
            if (previousDirection != direction)
            {
              levelsIndex++;
              prvUp = -INT_MAX;
            }

            if (curUp > prvUp)
            {
              levels[levelsIndex].index = i;
              levels[levelsIndex].up = prvUp = curUp;
            }
          }

          if (direction < 0) 
          {
            if (previousDirection != direction)
            {
              levelsIndex++;
              prvDn = INT_MAX;
            }

            if (curDn < prvDn)
            {
              levels[levelsIndex].index = i;
              levels[levelsIndex].down = prvDn = curDn;
            }
          }

          previousDirection = direction;
        }
      }

      return levelsIndex;
    }
};